@@ -9,12 +9,13 @@ import binascii
 import re
 import sys
 import types
+import warnings
 
-from ruamel.yaml.error import (MarkedYAMLError)
+from ruamel.yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning)
 from ruamel.yaml.nodes import *                               # NOQA
 from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
-                                ordereddict, text_type, nprint)
+                                ordereddict, text_type, nprint, version_tnf)
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
                                   CommentedKeySeq, CommentedSeq)
@@ -24,7 +25,7 @@ from ruamel.yaml.scalarstring import (PreservedScalarString, SingleQuotedScalarS
 from ruamel.yaml.scalarint import ScalarInt, BinaryInt, OctalInt, HexInt, HexCapsInt
 from ruamel.yaml.timestamp import TimeStamp
 
-if sys.version_info >= (3, 5, 2):
+if False:  # MYPY
     from typing import Any, Dict, List, Set, Generator  # NOQA
 
 
@@ -36,6 +37,14 @@ class ConstructorError(MarkedYAMLError):
     pass
 
 
+class DuplicateKeyFutureWarning(MarkedYAMLFutureWarning):
+    pass
+
+
+class DuplicateKeyError(MarkedYAMLFutureWarning):
+    pass
+
+
 class BaseConstructor(object):
 
     yaml_constructors = {}   # type: Dict[Any, Any]
@@ -44,7 +53,7 @@ class BaseConstructor(object):
     def __init__(self, preserve_quotes=None, loader=None):
         # type: (bool, Any) -> None
         self.loader = loader
-        if self.loader is not None:
+        if self.loader is not None and getattr(self.loader, '_constructor', None) is None:
             self.loader._constructor = self
         self.loader = loader
         self.constructed_objects = {}  # type: Dict[Any, Any]
@@ -52,22 +61,27 @@ class BaseConstructor(object):
         self.state_generators = []  # type: List[Any]
         self.deep_construct = False
         self._preserve_quotes = preserve_quotes
+        self.allow_duplicate_keys = version_tnf((0, 15, 1), (0, 16))
 
     @property
     def composer(self):
         # type: () -> Any
+        if hasattr(self.loader, 'typ'):
+            return self.loader.composer  # type: ignore
         try:
-            return self.loader._composer
+            return self.loader._composer  # type: ignore
         except AttributeError:
             print('slt', type(self))
-            print('slc', self.loader._composer)
+            print('slc', self.loader._composer)  # type: ignore
             print(dir(self))
             raise
 
     @property
     def resolver(self):
         # type: () -> Any
-        return self.loader._resolver
+        if hasattr(self.loader, 'typ'):
+            return self.loader.resolver  # type: ignore
+        return self.loader._resolver  # type: ignore
 
     def check_data(self):
         # type: () -> Any
@@ -117,7 +131,7 @@ class BaseConstructor(object):
                 None, None,
                 "found unconstructable recursive node", node.start_mark)
         self.recursive_objects[node] = None
-        constructor = None
+        constructor = None  # type: Any
         tag_suffix = None
         if node.tag in self.yaml_constructors:
             constructor = self.yaml_constructors[node.tag]
@@ -189,31 +203,63 @@ class BaseConstructor(object):
                 None, None,
                 "expected a mapping node, but found %s" % node.id,
                 node.start_mark)
-        mapping = {}
-        for key_node, value_node in node.value:
-            # keys can be list -> deep
-            key = self.construct_object(key_node, deep=True)
-            # lists are not hashable, but tuples are
-            if not isinstance(key, collections.Hashable):  # type: ignore
-                if isinstance(key, list):
-                    key = tuple(key)
-            if PY2:
-                try:
-                    hash(key)
-                except TypeError as exc:
-                    raise ConstructorError(
-                        "while constructing a mapping", node.start_mark,
-                        "found unacceptable key (%s)" %
-                        exc, key_node.start_mark)
-            else:
+        total_mapping = {}
+        if getattr(node, 'merge', None) is not None:
+            todo = [(node.merge, False), (node.value, False)]
+        else:
+            todo = [(node.value, True)]
+        for values, check in todo:
+            mapping = {}  # type: Dict[Any, Any]
+            for key_node, value_node in values:
+                # keys can be list -> deep
+                key = self.construct_object(key_node, deep=True)
+                # lists are not hashable, but tuples are
                 if not isinstance(key, collections.Hashable):
-                    raise ConstructorError(
-                        "while constructing a mapping", node.start_mark,
-                        "found unhashable key", key_node.start_mark)
+                    if isinstance(key, list):
+                        key = tuple(key)
+                if PY2:
+                    try:
+                        hash(key)
+                    except TypeError as exc:
+                        raise ConstructorError(
+                            "while constructing a mapping", node.start_mark,
+                            "found unacceptable key (%s)" %
+                            exc, key_node.start_mark)
+                else:
+                    if not isinstance(key, collections.Hashable):
+                        raise ConstructorError(
+                            "while constructing a mapping", node.start_mark,
+                            "found unhashable key", key_node.start_mark)
 
-            value = self.construct_object(value_node, deep=deep)
-            mapping[key] = value
-        return mapping
+                value = self.construct_object(value_node, deep=deep)
+                if check:
+                    self.check_mapping_key(node, key_node, mapping, key, value)
+                mapping[key] = value
+            total_mapping.update(mapping)
+        return total_mapping
+
+    def check_mapping_key(self, node, key_node, mapping, key, value):
+        # type: (Any, Any, Any, Any, Any) -> None
+        if key in mapping:
+            if not self.allow_duplicate_keys:
+                args = [
+                    "while constructing a mapping", node.start_mark,
+                    'found duplicate key "{}" with value "{}" '
+                    '(original value: "{}")'.format(
+                        key, value, mapping.get(key)), key_node.start_mark,
+                    """
+                    To suppress this check see:
+                        http://yaml.readthedocs.io/en/latest/api.html#duplicate-keys
+                    """,
+                    """\
+                    Duplicate keys will become and error in future releases, and are errors
+                    by default when using the new API.
+                    """,
+                ]
+                if self.allow_duplicate_keys is None:
+                    warnings.warn(DuplicateKeyFutureWarning(*args))
+                else:
+                    raise DuplicateKeyError(*args)
 
     def construct_pairs(self, node, deep=False):
         # type: (Any, bool) -> Any
@@ -295,6 +341,7 @@ class SafeConstructor(BaseConstructor):
             else:
                 index += 1
         if bool(merge):
+            node.merge = merge  # separate merge keys to be able to update without duplicate
             node.value = merge + node.value
 
     def construct_mapping(self, node, deep=False):
@@ -460,7 +507,7 @@ class SafeConstructor(BaseConstructor):
                 delta = -delta
         data = datetime.datetime(year, month, day, hour, minute, second,
                                  fraction)
-        if delta:  # type: ignore
+        if delta:
             data -= delta
         return data
 
@@ -1012,7 +1059,7 @@ class RoundTripConstructor(SafeConstructor):
             seqtyp._yaml_add_comment(node.comment[:2])
             if len(node.comment) > 2:
                 seqtyp.yaml_end_comment_extend(node.comment[2], clear=True)
-        if node.anchor:  # type: ignore
+        if node.anchor:
             from ruamel.yaml.serializer import templated_id
             if not templated_id(node.anchor):
                 seqtyp.yaml_set_anchor(node.anchor)
@@ -1101,10 +1148,10 @@ class RoundTripConstructor(SafeConstructor):
         merge_map = self.flatten_mapping(node)
         # mapping = {}
         if node.comment:
-            maptyp._yaml_add_comment(node.comment[:2])
+            maptyp._yaml_add_comment(node.comment[:2])  # type: ignore
             if len(node.comment) > 2:
-                maptyp.yaml_end_comment_extend(node.comment[2], clear=True)
-        if node.anchor:  # type: ignore
+                maptyp.yaml_end_comment_extend(node.comment[2], clear=True)  # type: ignore
+        if node.anchor:
             from ruamel.yaml.serializer import templated_id
             if not templated_id(node.anchor):
                 maptyp.yaml_set_anchor(node.anchor)
@@ -1113,7 +1160,7 @@ class RoundTripConstructor(SafeConstructor):
             # keys can be list -> deep
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
-            if not isinstance(key, collections.Hashable):  # type: ignore
+            if not isinstance(key, collections.Hashable):
                 if isinstance(key, list):
                     key_a = CommentedKeySeq(key)
                     if key_node.flow_style is True:
@@ -1136,6 +1183,8 @@ class RoundTripConstructor(SafeConstructor):
                         "while constructing a mapping", node.start_mark,
                         "found unhashable key", key_node.start_mark)
             value = self.construct_object(value_node, deep=deep)
+            self.check_mapping_key(node, key_node, maptyp, key, value)
+
             if key_node.comment and len(key_node.comment) > 4 and \
                key_node.comment[4]:
                 if last_value is None:
@@ -1143,21 +1192,21 @@ class RoundTripConstructor(SafeConstructor):
                     maptyp._yaml_add_comment(key_node.comment, value=last_key)
                 else:
                     key_node.comment[2] = key_node.comment.pop(4)
-                    maptyp._yaml_add_comment(key_node.comment, key=key)
+                    maptyp._yaml_add_comment(key_node.comment, key=key)  # type: ignore
                 key_node.comment = None
             if key_node.comment:
-                maptyp._yaml_add_comment(key_node.comment, key=key)
+                maptyp._yaml_add_comment(key_node.comment, key=key)  # type: ignore
             if value_node.comment:
-                maptyp._yaml_add_comment(value_node.comment, value=key)
-            maptyp._yaml_set_kv_line_col(
+                maptyp._yaml_add_comment(value_node.comment, value=key)  # type: ignore
+            maptyp._yaml_set_kv_line_col(  # type: ignore
                 key, [key_node.start_mark.line, key_node.start_mark.column,
                       value_node.start_mark.line, value_node.start_mark.column])
-            maptyp[key] = value
+            maptyp[key] = value  # type: ignore
             last_key, last_value = key, value  # could use indexing
         # do this last, or <<: before a key will prevent insertion in instances
         # of collections.OrderedDict (as they have no __contains__
         if merge_map:
-            maptyp.add_yaml_merge(merge_map)
+            maptyp.add_yaml_merge(merge_map)  # type: ignore
 
     def construct_setting(self, node, typ, deep=False):
         # type: (Any, Any, bool) -> Any
@@ -1170,7 +1219,7 @@ class RoundTripConstructor(SafeConstructor):
             typ._yaml_add_comment(node.comment[:2])
             if len(node.comment) > 2:
                 typ.yaml_end_comment_extend(node.comment[2], clear=True)
-        if node.anchor:  # type: ignore
+        if node.anchor:
             from ruamel.yaml.serializer import templated_id
             if not templated_id(node.anchor):
                 typ.yaml_set_anchor(node.anchor)
@@ -1178,7 +1227,7 @@ class RoundTripConstructor(SafeConstructor):
             # keys can be list -> deep
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
-            if not isinstance(key, collections.Hashable):  # type: ignore
+            if not isinstance(key, collections.Hashable):
                 if isinstance(key, list):
                     key = tuple(key)
             if PY2:
@@ -1194,7 +1243,9 @@ class RoundTripConstructor(SafeConstructor):
                     raise ConstructorError(
                         "while constructing a mapping", node.start_mark,
                         "found unhashable key", key_node.start_mark)
+            # construct but should be null
             value = self.construct_object(value_node, deep=deep)  # NOQA
+            self.check_mapping_key(node, key_node, typ, key, value)
             if key_node.comment:
                 typ._yaml_add_comment(key_node.comment, key=key)
             if value_node.comment:
@@ -1327,7 +1378,7 @@ class RoundTripConstructor(SafeConstructor):
             delta = datetime.timedelta(hours=tz_hour, minutes=tz_minute)
             if values['tz_sign'] == '-':
                 delta = -delta
-        if delta:  # type: ignore
+        if delta:
             dt = datetime.datetime(year, month, day, hour, minute)
             dt -= delta
             data = TimeStamp(dt.year, dt.month, dt.day, dt.hour, dt.minute,
